/*******************************************
Stata Short Course
2021 CEMFI Undergraduate Summer Internship
By: Micole De Vera
*******************************************/

clear

* There are a few ways to comment on do-files

set more off // Some can be used beside codes

/* For longer comments...
or if you want to comment out a chunk of code...
you can use this */

** Set the working directory
cd "C:\Users\micol\Desktop\USI_2021"

/********************************************
PART I: DESCRIBING DATA
********************************************/
** Start a log file
log using "log\USI_stata_part1.smcl", replace

** Let's start with the data in 2011
use "raw\data_2011.dta", clear

** How to save?
save "dta\data_2011_copy.dta", replace

** Let's see how the data looks like
describe
br
br person_id occ
br person_id occ if sex == 1 	// Males

** Summary statistics
sum
sum if sex == 1
sum wage, detail

tab age
tab age sex
tab age sex, column		// With frequencies

tabstat wage, by(age) stat(mean sd)
tab age sex, summarize(wage)

count if age > 40

** Correlations
corr age yrsexp wage
pwcorr age yrsexp wage, obs

** Test of means
ttest wage, by(sex)

** Accessing results 
sum age, detail
return list
display r(skewness)

** Drop and keep
* Suppose we want to only keep observations of individuals between
* ages of 40 and 55
keep if age >= 40
drop if age > 55

** Close log file
log close
translate "log\USI_stata_part1.smcl" "log\USI_stata_part1.pdf", replace

/*********************************************
PART II: APPEND, MERGE, COLLAPSE, and RESHAPE
*********************************************/
** Suppose we want to combine the data from different years
use "raw\data_2011.dta", clear
append using "raw\data_2009.dta"

save "dta\data_09_11.dta", replace

** Merging data: We have occupation-specific data we want to combine
* In the merging data, we have info on occupations at 1-digit level
merge m:1 occ1 using "raw\occ_info.dta"

** Collapsing data
* Suppose we want to make a new dataset made of average wages
* for men and women over time
collapse (mean) mean_wage=wage educ (sum) sum_wage=wage age, by(year sex)

** Reshaping data
use "dta\data_09_11.dta", replace

* Our data is currently in long format. Let's transform it to long format.
* long format - each individual-year is a separate observation
* wide format - each individual is an observation, each yearly value is a separate var
reshape wide wage age yrsexp occ1, i(person_id) j(year) // This won't work because occ is not constant across time

drop occ
reshape wide wage age yrsexp occ1, i(person_id) j(year)

save "dta\data_09_11_wide.dta", replace

* Let's make it back to long
reshape long wage age yrsexp occ1, i(person_id) j(year)

/*********************************************
PART III: OPERATIONS WITH DATA
*********************************************/
use "raw\data_2011.dta", clear

** Generating new variables
* Suppose we want to make a variable that is age squared
gen age_sq = age^2

* Indicator for female?
gen female = (sex == 2)

* Log wages?
gen ln_wage = log(wage)

* Average log wage for males and females?
egen ave_lnwage_m = mean(ln_wage) if sex == 1
egen ave_lnwage_f = mean(ln_wage) if sex == 2

* Let's make a variable that measures difference of indiv wage to sex-specific average
gen wage_dev = ln_wage - ave_lnwage_m if sex == 1
replace wage_dev = ln_wage - ave_lnwage_f if sex == 2

* We can also use outputs from summarize or tab to make variables
* Nonsense example:
sum age, detail
gen nonsense = age - r(skewness)

** Labels and label values
* Let's label a variable
label var female "=1 if female"

* Let's also give it a value label
label define fem_lab 0 "Male" 1 "Female"
label values female fem_lab

** Working with strings
* Let's make a variable that is a string
gen birth_year = year - age
tostring birth_year, replace

* What does it mean?
gen newvar = birth_year - 1900 	// We cannot do math operations on strings

* Let's try to get the last two digits (as a string)
gen birth_year_2dig = substr(birth_year, 3, 2)
destring birth_year_2dig, replace

** Missing values
* Note that wage has missing values
count if missing(wage)

* Let's generate a variable that indicates if the individual is a top-earner
sum wage, d
gen topearner = (wage >= r(p99))

* What did this do to missing values?
br wage topearner if missing(wage)

* How to fix it?
sum wage, d
gen topearner_fixed = (wage >= r(p99)) if !missing(wage)
gen topearner_fixed2 = (wage >= r(p99)) if wage != . 

** Calling variables
* Let's make variables only useful for this exercise
gen v1 = 1
gen v2 = 2
gen v3 = 3
gen v4 = 4
gen v5 = 5
gen v6 = 6
gen v10 = 10
gen v100 = 100
gen v1000 = 1000
gen v10000 = 10000

sum v1-v3 v5
sum v*
sum v?
sum v??

* How about with real data
use "dta\data_09_11_wide.dta", clear

sum age*
sum age2009-age2011

** Working with by and bysort
use "raw\data_2011.dta", clear

* Recall: computing wage deviation from gender-specific average
gen ln_wage = log(wage)

* Average log wage for males and females?
egen ave_lnwage_m = mean(ln_wage) if sex == 1
egen ave_lnwage_f = mean(ln_wage) if sex == 2

* Let's make a variable that measures difference of indiv wage to sex-specific average
gen wage_dev = ln_wage - ave_lnwage_m if sex == 1
replace wage_dev = ln_wage - ave_lnwage_f if sex == 2

* Let us do the same using bysort
bysort sex: egen group_mean = mean(ln_wage)
gen wage_dev2 = ln_wage - group_mean

gen check = (wage_dev == wage_dev2)
tab check

/*********************************************
PART IV: OTHER TOOLS
*********************************************/
** _n and _N
use "dta\data_09_11.dta", clear

* We can count how many times we observe an individual
bysort person_id: gen total_obs = _N

* We can label the observations in chronological order
bysort person_id (year): gen counter = _n

* Statistics
tab total_obs if counter == 1

* Let's check if age difference between 2009 and 2011 is exactly 2 years
* This is a very strict consistency check because there are real possibilities why
* this does not happen.
gen flag_age = 0
bysort person_id (year): replace flag_age = 1 if _n > 1 & age[_n] - age[_n-1] != year[_n] - year[_n-1]
bysort person_id: egen flag_age_tot = sum(flag_age)

** Locals and globals
* Use a local to store a number or string
local fun = 0
gen new_var = `fun'
gen new_var2 = `fun'

* Same for globals
global fun1 = 0
gen new_var1 = $fun1
gen new_var2 = $fun1

* Locals and globals can store more than one object and this is useful later on
* when we look at loops
gen age40 = cond(age == 40, 1, 0)

** Loops
* forvalues
forvalues y = 0(5)40 {
	gen var_`y' = `y'
}

* foreach
foreach n in 1 3 5 7 9 {
	di `n'
}

* You can combine locals/globals with foreach 
local years 1995 2000 2005 2007 2015
foreach yr in `years' {
	di `yr'
}

* Something more complicated
local codes `""A" "B" "C""'
local counter = 1
foreach c in `codes' {
	gen var_`c' = `counter'
	local counter = `counter' + 1
}

/*********************************************
PUTTING IT ALL TOGETHER
*********************************************/
* GOAL: difference in average log wages by sex over time
* Step 1: Append data across years
local years 1990 1999 2007 2009 2011

use "raw\data_1981.dta", clear
foreach y in `years' {
	append using "raw\data_`y'.dta"
}

save "dta\data_all.dta", replace

* Step 2: Generate log wages
gen ln_wage = log(wage)

* Step 3: Collapse by sex and year
collapse (mean) ln_wage, by(sex year)

* Step4: Reshape to have male and female average ln_wage as separate columns
reshape wide ln_wage, i(year) j(sex)

* Step5: Gender wage gap (ln_wage_male - ln_wage_female)
gen wage_gap = ln_wage1 - ln_wage2

save "out\raw_wage_gaps.dta", replace

/*********************************************
PART V: LINEAR REGRESSION
*********************************************/
use "dta\data_all.dta", clear

** Simple Mincerian regression
gen ln_wage = log(wage)
gen yrsexp_sq = yrsexp^2

reg ln_wage educ yrsexp yrsexp_sq

* Gender wage gap?
gen female = (sex == 2)

reg ln_wage female
reg ln_wage female educ yrsexp yrsexp_sq

** Computing standard errors differently
reg ln_wage female educ yrsexp yrsexp_sq
reg ln_wage female educ yrsexp yrsexp_sq, vce(robust)
reg ln_wage female educ yrsexp yrsexp_sq, vce(cluster person_id)

** Let's now talk about factor variables and interactions
* Suppose we had a different education variable
gen educ_group = .
replace educ_group = 1 if educ < 12 	// Less than HS
replace educ_group = 2 if educ >= 12 & educ < 16	// Some college
replace educ_group = 3 if educ > 16 & !missing(educ) // College degree or higher

* Now, the numbers 1,2,3 in educ_group is just meant to represent some ordering
reg ln_wage female i.educ_group yrsexp yrsexp_sq

* What if returns to education might be different by gender?
reg ln_wage female i.educ_group i.female#i.educ_group yrsexp yrsexp_sq
reg ln_wage i.female##i.educ_group yrsexp yrsexp_sq

* What if both returns to education and returns to experience are different by gender?
reg ln_wage i.female##(i.educ_group c.yrsexp c.yrsexp_sq)

* We can use ## to form squares
reg ln_wage female educ c.yrsexp##c.yrsexp

** Using locals and globals to store controls
global controls educ c.yrsexp##c.yrsexp
reg ln_wage $controls, vce(cluster person_id)
reg ln_wage female $controls

** With panel data, we have the opportunity to control for 
** time invariant unobserved heterogeneity
* Let us declare the data as a panel
xtset person_id year

* Mincer regression with FE
xtreg ln_wage educ c.yrsexp##c.yrsexp, fe vce(cluster person_id)

* Once we have declared the data as a panel, we can construct lags easily
* Unfortunately, since our data is not sequential, this is not too useful here
gen wage_lag = l.ln_wage
gen wage_lag2 = l2.ln_wage

reg ln_wage l2.ln_wage

** Accessing results of a regression?
reg ln_wage female educ c.yrsexp##c.yrsexp
ereturn list

gen preds = _b[_cons] + _b[female] * female + _b[educ] * educ ///
			+ _b[yrsexp] * yrsexp + _b[c.yrsexp#c.yrsexp] * yrsexp^2 
gen resids = ln_wage - preds if e(sample)

predict preds2 if e(sample)
predict resids2 if e(sample), residuals

gen check_pred = preds - preds2
gen check_resid = resids - resids2
sum check_pred check_resid

** Saving the results of a regression
* ssc install outreg2, replace
reg ln_wage female
outreg2 using "out\regressions", tex replace ctitle(raw)

reg ln_wage female educ
outreg2 using "out\regressions", tex append ctitle(educ)

reg ln_wage female educ c.yrsexp##c.yrsexp
outreg2 using "out\regressions", tex append ctitle(educ+exp)

/*********************************************
PART VI: MAKING GRAPHS
*********************************************/
use "dta\data_all.dta", clear

** Histograms and kernel densities
histogram wage
kdensity wage

gen ln_wage = log(wage)
histogram ln_wage
kdensity ln_wage, normal

histogram educ
histogram educ, discrete

** Twoway plots
preserve

sample 1
twoway (scatter ln_wage age) (lfit ln_wage age)

restore

twoway (kdensity ln_wage if sex == 1) ///
		(kdensity ln_wage if sex == 2), ///
		legend(label(1 "Male") label(2 "Female"))
		
* Let us save this graph
graph export "figs\wage_distn_bysex.pdf", as(pdf) replace

/*********************************************
COMBINING IDEAS
*********************************************/
* GOAL: A figure with gender gaps across years (raw and controlling for Mincer controls)
use "dta\data_all.dta", clear

* Step 1: Prepare a matrix to store regression estimates
tab year
matrix gender_gaps = J(r(r), 3, 0)

* Step 2: Do the regressions and store coefficients
gen ln_wage = log(wage)
gen female = (sex == 2)

levelsof year, local(yr_levels)
di `yr_levels'
local counter = 1
foreach y in `yr_levels' {
	* First column of result matrix is year
	matrix gender_gaps[`counter', 1] = `y'
	
	* Regression to get raw gender gap
	reg ln_wage female if year == `y'
	matrix gender_gaps[`counter', 2] = _b[female]
	
	* Regression to get gender gap controlling education and experience
	reg ln_wage female educ c.yrsexp##c.yrsexp if year == `y'
	matrix gender_gaps[`counter', 3] = _b[female]
	
	* Update counter
	local counter = `counter' + 1
}

matrix list gender_gaps	 	// Let's see what we have

* Step 3: Let's make the matrix into a dataset
drop _all

svmat gender_gaps

rename gender_gaps1 year		// An alternative to this is to define the colnames of the matrix
rename gender_gaps2 gap_raw
rename gender_gaps3 gap_cont

* Step 4: Graph
twoway (connected gap_raw year) (connected gap_cont year), ///
			title("Gender Gaps", span) ///
			xtitle("Year") ytitle("Gender Gap") ///
			legend(label(1 "No Controls") label(2 "Mincer Controls") ///
				col(2)) ///
			graphregion(color(white))
graph export "figs\gender_gaps.pdf", as(pdf) replace		












